#!/bin/bash

#Check if the flags "u" or "e" is already set
RESET_FLAG_U_REQUIRED=false; RESET_FLAG_E_REQUIRED=false;
if [[ ! "$-" =~ u ]]; then RESET_FLAG_U_REQUIRED=true;  set -u; fi
if [[ ! "$-" =~ e ]]; then RESET_FLAG_E_REQUIRED=true;  set -e; fi

script=${BASH_SOURCE[0]:-${(%):-%x}}
bin=`dirname "$script"`
bin=`cd "$bin"; pwd`

function err_pr {
	echo "[ERROR] $*" >&2
	exit 1
}
function print_nodelist {
	scontrol show hostname $SLURM_NODELIST
}
function get_config_keys {
	set | sed -n -e 's/^\(FRAMEWORK_[^=]*\).*/\1/p'
}
function expand {
	[[ "$SHELL" =~ zsh ]] && echo ${(P)1}
	[[ "$SHELL" =~ bash ]] && echo ${!1}
}
function create_configuration_dir {
local FW_NAME=$1
local FW_TEMPLATE_CONF_DIR=$2
local FRAMEWORK_CONF_DIR=${3:-"$HOME/cluster-conf-$SLURM_JOB_ID"}
FRAMEWORK_CONF_DIR="$(realpath $FRAMEWORK_CONF_DIR)/${FW_NAME}"
# include framework specific stuff here, such as files to modify (FRAMEWORK_CONF_FILES), file to enumerate slaves (FRAMEWORK_SLAVE_FILE), file to enumerate masters (FRAMEWORK_MASTER_FILE), path to scripts (FRAMEWORK_BIN_DIR)
local META_CONF=$FW_TEMPLATE_CONF_DIR/meta.conf
if [ -e "$META_CONF" ]; then
. "$META_CONF"
elif [ -e "$bin/configs/$FW_NAME" ]; then
. "$bin/configs/$FW_NAME"
else
err_pr "No file 'meta.conf' in $FW_TEMPLATE_CONF_DIR"
fi
local NODES=$SLURM_JOB_NUM_NODES
local FRAMEWORK_CPUS_PER_NODE=${SLURM_JOB_CPUS_PER_NODE:-1}
local FRAMEWORK_MEM_PER_NODE=${SLURM_MEM_PER_NODE:-61440}m
local FRAMEWORK_CONF_BASE=`dirname $FRAMEWORK_CONF_DIR`
mkdir -p "$FRAMEWORK_CONF_BASE"
cp -Tr "$FW_TEMPLATE_CONF_DIR" "$FRAMEWORK_CONF_DIR"
chmod -R u+w "$FRAMEWORK_CONF_BASE"
local FRAMEWORK_LOG_DIR=${FRAMEWORK_LOG_DIR:-$FRAMEWORK_CONF_DIR/log}
local FRAMEWORK_PID_DIR=${FRAMEWORK_PID_DIR:-$FRAMEWORK_CONF_DIR/pid}
local FRAMEWORK_LOCAL_DIR=/tmp/$USER/cluster-conf-$SLURM_JOB_ID
local FRAMEWORK_DATA_DIR=${FRAMEWORK_DATA_DIR:-$FRAMEWORK_LOCAL_DIR}
mkdir -p "$FRAMEWORK_LOG_DIR"
mkdir -p "$FRAMEWORK_PID_DIR"
local FRAMEWORK_MASTER_NODE=$(print_nodelist | awk '{print $1}' | sort -u | head -n 1)
if [ 1 -eq $NODES ]; then
local FRAMEWORK_SLAVE_NODES=$FRAMEWORK_MASTER_NODE
echo "$FRAMEWORK_SLAVE_NODES" > "$FRAMEWORK_CONF_DIR/$FRAMEWORK_SLAVE_FILE"
else
local FRAMEWORK_SLAVE_NODES=$(print_nodelist | awk '{print $1}' | sort -u | sed ":a;N;s/\\n/,/g;ba")
print_nodelist | awk '{print $1}' | sort -u > "$FRAMEWORK_CONF_DIR/$FRAMEWORK_SLAVE_FILE"
fi
if [ -f "${FRAMEWORK_SLAVE_FILE_ALTS+/does/not/exist}" ]; then
for altSlave in $FRAMEWORK_SLAVE_FILE_ALTS; do
[ -f "$FRAMEWORK_CONF_DIR/$altSlave" ] && rm "$FRAMEWORK_CONF_DIR/$altSlave"
ln -s "$FRAMEWORK_CONF_DIR/$FRAMEWORK_SLAVE_FILE" "$FRAMEWORK_CONF_DIR/$altSlave"
done
fi
#persist keyword replacements:
cat <<EOF > "$FRAMEWORK_CONF_DIR/config-subs.txt"
NODES=$NODES
FRAMEWORK_MASTER_NODE="$FRAMEWORK_MASTER_NODE"
FRAMEWORK_SLAVE_NODES="$FRAMEWORK_SLAVE_NODES"
FRAMEWORK_CONF_DIR="$FRAMEWORK_CONF_DIR"
FRAMEWORK_LOG_DIR="$FRAMEWORK_LOG_DIR"
FRAMEWORK_PID_DIR="$FRAMEWORK_PID_DIR"
FRAMEWORK_CONF_FILES="$FRAMEWORK_CONF_FILES"
FRAMEWORK_JAVA_HOME="$JAVA_HOME"
FRAMEWORK_CPUS_PER_NODE="$FRAMEWORK_CPUS_PER_NODE"
FRAMEWORK_MEM_PER_NODE="$FRAMEWORK_MEM_PER_NODE"
FRAMEWORK_BIN_DIR="$FRAMEWORK_BIN_DIR"
FRAMEWORK_DATA_DIR="$FRAMEWORK_DATA_DIR"
FRAMEWORK_LOCAL_DIR="$FRAMEWORK_LOCAL_DIR"
EOF
if [ "x$FRAMEWORK_MASTER_FILE" != "x" ]; then
	echo "$FRAMEWORK_MASTER_NODE" > "$FRAMEWORK_CONF_DIR/$FRAMEWORK_MASTER_FILE"
fi
#read keyword replacements
. $FRAMEWORK_CONF_DIR/config-subs.txt
#replace keywords:
for FRAMEWORK_FILE in `echo $FRAMEWORK_CONF_FILES`
do
	if [ -f "$FRAMEWORK_CONF_DIR/$FRAMEWORK_FILE" ]; then
		for KEY in `get_config_keys`; do
			sed -i "s#$KEY#`expand $KEY`#g" "$FRAMEWORK_CONF_DIR/$FRAMEWORK_FILE"
		done
	fi
done
local i=0
if [ "x$FRAMEWORK_DATA_DIR" = "x$FRAMEWORK_LOCAL_DIR" ]; then
	for node in $FRAMEWORK_MASTER_NODE $(cat $FRAMEWORK_CONF_DIR/$FRAMEWORK_SLAVE_FILE)
	do
		srun --overlap -N1 -n1 --nodelist=$node bash -c "mkdir -p $FRAMEWORK_LOCAL_DIR"
	done
else
	for node in $FRAMEWORK_MASTER_NODE $(cat $FRAMEWORK_CONF_DIR/$FRAMEWORK_SLAVE_FILE)
	do
		if [ ! -d $FRAMEWORK_DATA_DIR/$i ]; then
			srun --overlap -N1 -n1 --nodelist=$node bash -c "mkdir -p $(dirname $FRAMEWORK_LOCAL_DIR); mkdir -p $FRAMEWORK_DATA_DIR/$i; [ ! -d $FRAMEWORK_LOCAL_DIR ] && ln -s -T $FRAMEWORK_DATA_DIR/$i $FRAMEWORK_LOCAL_DIR"
		fi
		let i++
	done
fi
export `echo ${FW_NAME}_CONF_DIR | tr '[:lower:]' '[:upper:]'`=$FRAMEWORK_CONF_DIR
}

usage() {
cat <<-EOF
Usage: source framework-configure.sh <framework-name> <config-template-dir> [config-dst-dir]
  framework-name      : Name of the framework whose configuration
                        is required.
  config-template-dir : Configuration template directory.
  config-dst-dir      : Custom location for configuration
                        files. If not provided, files will
                        be placed in home directory.
EOF
}

if [[ $# -eq 1 ]] && ( [[ $1 == "-h" ]] || [[ $1 == "--help" ]] ); then
    usage
elif [[ $# -eq 2 ]]; then
    FW_NAME=$1
    FW_TEMPLATE_CONF_DIR=$2
    if [ -d "$FW_TEMPLATE_CONF_DIR" ]; then
        create_configuration_dir "$FW_NAME" "$FW_TEMPLATE_CONF_DIR"
    else
        err_pr "Please provide a valid directory name as second argument"
    fi
elif [[ $# -eq 3 ]]; then
    FW_NAME=$1
    FW_TEMPLATE_CONF_DIR=$2
    FW_CONFIG_DST=$3
    if [ -d "$FW_TEMPLATE_CONF_DIR" ]; then
        create_configuration_dir "$FW_NAME" "$FW_TEMPLATE_CONF_DIR" "$FW_CONFIG_DST"
    else
        err_pr "Please provide a valid directory name as second argument"
    fi
else
    err_pr "Incorrect usage. Try \"framework-configure.sh --help\" for more information"
fi

if $RESET_FLAG_U_REQUIRED; then set +u; fi
if $RESET_FLAG_E_REQUIRED; then set +e; fi
